# Sprites for Games

import pygame
import random
import math

# Provides a sprite that can be moved around the screen
# The sprite can be told to move in a particular direction
# and will move in that direction each time update is called
# The sprite will not leave the screen area
# Ihe sprite will bounce off the edges and the move commands
# will cause acceleration

class Cheese:

    # Initialise the Mover object
    #    position - the initial position of the player
    #    limit - a tuple giving the size of the screen
    #    accel - a tuple giving the acceleration in pixels/second/second
    #    image - the image to be drawn
    #    scale - the x size of the image in pixels
    def __init__(self, position, limit, accel, image, width):
        
        # record the initial position as the reset position
        self.resetPosition = position

        # Scale and store the image in a property called image
        # Scale and store the image in a property called image
        aspect = image.get_width()/image.get_height()
        height = width * aspect
        self.image = pygame.transform.scale(image,(int(width),int(height)))
        
        # Create a rectangle which is the size of the player
        # We use the dimensions of the image to work out the player size
        self.rect = pygame.Rect(0,0,self.image.get_width(), self.image.get_height())

        # Copy the screen size limits into a local variable
        self.limit = [float(limit[0]), float(limit[1])]

        # Set the movement speed of the player to 200 pixels per second
        # In a later version we might have different player speeds
        self.movementSpeed = [0,0] # pixels per second
        self.acceleration = [float(accel[0]), float(accel[1])] # pixels per second per second
        self.friction = [.6,.6] # speed reduction over a second 

        # Reset the player
        self.reset()
                    
    # Draws the player on the screen at its current position
    def draw(self, surface):
        surface.blit(self.image, self.position)

    # Updates the state of the player
    #     deltaTime - the time in seconds since update was last called
    #                 usually (but not always) a 120th of a second
    def update(self, deltaTime):
        
        # Handle movement - if we are moving in that direction

        # Update the speed using the accelleration 
        if self.movingUp:
            self.movementSpeed[1] = self.movementSpeed[1] - (self.acceleration[1]*deltaTime)
        if self.movingDown:
            self.movementSpeed[1] = self.movementSpeed[1] + (self.acceleration[1]*deltaTime)
        if self.movingLeft:
            self.movementSpeed[0] = self.movementSpeed[0] - (self.acceleration[0]*deltaTime)
        if self.movingRight:
            self.movementSpeed[0] = self.movementSpeed[0] + (self.acceleration[0]*deltaTime)

        # Impose the friction
        
        self.movementSpeed[0] = self.movementSpeed[0] * (1 - deltaTime * self.friction[0])
        self.movementSpeed[1] = self.movementSpeed[1] * (1 - deltaTime * self.friction[1])

        # Clamp the speed
        if math.fabs(self.movementSpeed[0]) < .01:
            self.movementSpeed[0] = 0
        if math.fabs(self.movementSpeed[1]) < .01:
            self.movementSpeed[1] = 0

        # change the current position by the speed value
        self.position[0] = self.position[0] + self.movementSpeed[0]
        self.position[1] = self.position[1] + self.movementSpeed[1]

        # Clamp the position values so we can't move off the screen

        if self.position[0] < 0:
            self.position[0]=0
            self.movementSpeed[0] = math.fabs(self.movementSpeed[0])
        if self.position[1] < 0:
            self.position[1]=0
            self.movementSpeed[1] = math.fabs(self.movementSpeed[1])
        if self.position[0] + self.image.get_width() > self.limit[0]:
            self.position[0] = self.limit[0] - self.image.get_width()
            self.movementSpeed[0] = math.fabs(self.movementSpeed[0])*-1
        if self.position[1] + self.image.get_height() > self.limit[1]:
            self.position[1] = self.limit[1] - self.image.get_height()
            self.movementSpeed[1] = math.fabs(self.movementSpeed[1])*-1

        # Move to the new position

        self.setPosition(self.position)

    # Movement controllers - two for each direction
    
    def StartMoveUp(self):
        self.movingUp = True
    def StopMoveUp(self):
        self.movingUp = False
        
    def StartMoveDown(self):
        self.movingDown = True
    def StopMoveDown(self):
        self.movingDown = False

    def StartMoveLeft(self):
        self.movingLeft = True
    def StopMoveLeft(self):
        self.movingLeft = False

    def StartMoveRight(self):
        self.movingRight = True
    def StopMoveRight(self):
        self.movingRight = False

    # Sets the position of the sprite on the screen
    # Also updates the rectangle that defines the screen area
    # occupied by the sprite
    def setPosition(self,position):
        self.position = [float(position[0]), float(position[1])]
        self.rect[0]= self.position[0]
        self.rect[1]= self.position[1]

    # Sets the movement speed of the sprite
    #     speed - tuple containing positive x and y speed values
    #             
    def setSpeed(self,speed):
        self.movementSpeed = [float(speed[0]), float(speed[1])]

    # Resets the state of the sprite for a new game
    # Puts it back to start position and clears all the
    # movement flags
    def reset(self):
        self.setPosition(self.resetPosition)
        self.movementSpeed=[0,0]
        self.movingUp = False
        self.movingDown = False
        self.movingLeft = False
        self.movingRight = False
        self.cheeseDead = False
               
    # Gets the name of this sprite
    def getName(self):
        return "mover"

    # Gets the description of this sprite
    def getDescription(self):
        return "A moving sprite that will not leave the playfield"
       
    # Gets the centre of this sprite as a tuple
    def getCentre(self):
        x = self.position[0] + (self.image.get_width()/2)
        y = self.position[1] + (self.image.get_height()/2)
        return (x,y)

    # Gets the distance from a sprite
    def getDistanceFrom(self, s):
        p1 = self.getCentre()
        p2 = s.getCentre()
        dx = p1[0] - p2[0]
        dy = p1[1] - p2[1]
        return math.sqrt((dx*dx) + (dy*dy))

